unit INIReaderWriter;

interface

uses
  System.Classes, System.IniFiles;

type
  INIAttribute = class(TCustomAttribute)
  private
    FSection: string;
    FValue: string;
  public
    property Section: string read FSection;
    property Value: string read FValue;
    constructor Create(const ASection, AValue: string);
  end;

  TObjectIniReaderWriter = class(TObject)
  public
    class procedure Load(const AIniFile: TCustomIniFile; AObject: TObject); overload; static;
    class procedure Load(const AIniFile: string; AObject: TObject); overload; static;
    class procedure Save(const AIniFile: TCustomIniFile; AObject: TObject); overload; static;
    class procedure Save(const AIniFile: string; AObject: TObject); overload; static;
  end;

implementation

uses
  System.Variants, System.TypInfo, System.SysUtils, System.Rtti;

{ INIAttribute }

constructor INIAttribute.Create(const ASection, AValue: string);
begin
  inherited Create;
  FSection := ASection;
  FValue := AValue;
end;

{ TObjectIniReaderWriter }

class procedure TObjectIniReaderWriter.Load(const AIniFile: TCustomIniFile; AObject: TObject);
var
  LCtx: TRttiContext;
  LType: TRttiType;
  LProps: TArray<TRttiProperty>;
  LProp: TRttiProperty;
  LAttrs: TArray<TCustomAttribute>;
  LAttr: TCustomAttribute;
  LSection, LValue: string;
  LDefault: Variant;
  LDefInt: Integer;
  LDefStr: string;
  LDefBool: Boolean;
  LDefFloat: Extended;
  LDefCurr: Currency;
begin
  LCtx := TRttiContext.Create;
  LType := LCtx.GetType(AObject.ClassType);
  LProps := LType.GetProperties;
  for LProp in LProps do
  begin
    LSection := '';
    LValue := '';
    LDefault := Unassigned;

    LAttrs := LProp.GetAttributes;
    for LAttr in LAttrs do
      if LAttr is INIAttribute then
      begin
        LSection := INIAttribute(LAttr).Section;
        LValue := INIAttribute(LAttr).Value;
      end
      else if LAttr is DefaultAttribute then
        LDefault := DefaultAttribute(LAttr).Value;

    if LValue <> '' then
      case LProp.PropertyType.TypeKind of
        tkInteger,
        tkInt64:
          begin
            LDefInt := 0;
            if not VarIsEmpty(LDefault) then
              LDefInt := LDefault;
            LProp.SetValue(AObject, AIniFile.ReadInteger(LSection, LValue, LDefInt));
          end;
        tkChar,
        tkWChar,
        tkString,
        tkUString:
          begin
            LDefStr := '';
            if not VarIsEmpty(LDefault) then
              LDefStr := LDefault;
            LProp.SetValue(AObject, AIniFile.ReadString(LSection, LValue, LDefStr));
          end;
        tkEnumeration:
          if LProp.PropertyType.Handle = System.TypeInfo(Boolean) then
          begin
            LDefBool := False;
            if not VarIsEmpty(LDefault) then
              LDefBool := LDefault;
            LProp.SetValue(AObject, AIniFile.ReadBool(LSection, LValue, LDefBool));
          end;
        tkFloat:
          if GetTypeData(LProp.PropertyType.Handle)^.FloatType = ftCurr then
          begin
            LDefCurr := 0;
            if not VarIsEmpty(LDefault) then
              LDefCurr := LDefault;
            LProp.SetValue(AObject, StrToCurr(AIniFile.ReadString(LSection, LValue, CurrToStr(LDefCurr))));
          end
          else
          begin
            LDefFloat := 0;
            if not VarIsEmpty(LDefault) then
              LDefFloat := LDefault;
            LProp.SetValue(AObject, AIniFile.ReadFloat(LSection, LValue, LDefFloat));
          end;
      end;
  end;
end;

class procedure TObjectIniReaderWriter.Load(const AIniFile: string; AObject: TObject);
var
  LIni: TIniFile;
begin
  LIni := TIniFile.Create(AIniFile);
  try
    Load(LIni, AObject);
  finally
    LIni.Free;
  end;
end;

class procedure TObjectIniReaderWriter.Save(const AIniFile: TCustomIniFile; AObject: TObject);
var
  LCtx: TRttiContext;
  LType: TRttiType;
  LProps: TArray<TRttiProperty>;
  LProp: TRttiProperty;
  LAttrs: TArray<TCustomAttribute>;
  LAttr: TCustomAttribute;
  LSection, LValue: string;
begin
  LCtx := TRttiContext.Create;
  LType := LCtx.GetType(AObject.ClassType);
  LProps := LType.GetProperties;
  for LProp in LProps do
  begin
    LSection := '';
    LValue := '';
    LAttrs := LProp.GetAttributes;
    for LAttr in LAttrs do
      if LAttr is INIAttribute then
      begin
        LSection := INIAttribute(LAttr).Section;
        LValue := INIAttribute(LAttr).Value;
      end;
    if LValue <> '' then
      case LProp.PropertyType.TypeKind of
        tkInteger,
        tkInt64:
          AIniFile.WriteInteger(LSection, LValue, LProp.GetValue(AObject).AsInt64);
        tkString,
        tkUString:
          AIniFile.WriteString(LSection, LValue, LProp.GetValue(AObject).AsString);
        tkEnumeration:
          if LProp.PropertyType.Handle = System.TypeInfo(Boolean) then
            AIniFile.WriteBool(LSection, LValue, LProp.GetValue(AObject).AsBoolean);
        tkFloat:
          if GetTypeData(LProp.PropertyType.Handle)^.FloatType = ftCurr then
            AIniFile.WriteString(LSection, LValue, CurrToStr(LProp.GetValue(AObject).AsCurrency))
          else
            AIniFile.WriteFloat(LSection, LValue, LProp.GetValue(AObject).AsExtended);
      end;
  end;
end;

class procedure TObjectIniReaderWriter.Save(const AIniFile: string; AObject: TObject);
var
  LIni: TIniFile;
begin
  LIni := TIniFile.Create(AIniFile);
  try
    Save(LIni, AObject);
  finally
    LIni.Free;
  end;
end;

end.

